# Functions for the supplementary runs where each host
# is infected with one parasite

rm(list=ls())
source("Recombination Dec 2011 Dryad.R")
source("parameters Dec 2011 Dryad.R")
source("Setting up Pop'n Dec 2011 Dryad.R")

obtain.A <- function(nh, xp, mu.h, mu.p, alpha.h, alpha.p, beta, ra, rb, rc, R, num) {

	output.h <- matrix(NA, when.intro, length(nh))
	output.p <- matrix(NA, when.intro)
	
	i <- 0
	
	repeat {
		i <- i + 1
		
		xh <- nh/num
#		xp <- xp/num
		
# Each host gets a single parasite, drawn from the effectively infinite pool of parasite propagules:
        	typeP <- rbinom(8,size=nh,prob=xp)
        	typep <- nh-typeP

#The frequency of P carried by each host genotype is
	    freqP <- typeP/(typeP+typep)

#Replacing NA with 0 for those host genotypes that don't exist
        freqP[is.na(freqP)] <- 0
 
#The average fitness of hosts of a particular genotype depends on the frequency of P actually within that genotype:
#Recall that matching reduces host fitness (alpha.h is negative).  With this haploid, single host-parasite locus model,
#MAM and IMAM can be obtained simply by switching the labels of H/h.

        	wh <- (1 + alpha.h * (gtypes$A * freqP + (1 - gtypes$A) * (1 -freqP))) * (1 + (gtypes$B) * beta)
		xhtemp <- next.gen.h(xh, wh)
		
#The fraction of parasites that match their host for both P and p parasites:

    	    matchP <- sum(gtypes$A*typeP)/sum(typeP)
        	matchp <- sum((1-gtypes$A)*typep)/sum(typep)

#Replacing NA with 0 for those parasite genotypes that don't exist
        matchP[is.na(matchP)] <- 0
        matchp[is.na(matchp)] <- 0
 
#Then the average fitness of each parasite genotype, accounting for the fitness advantage of infection via matching is:

        	wp <- 1 + alpha.p * c(matchP, matchp)

#These num parasites (one in each host) then produce an effectively infinite number of parasite propagules at frequency:

        xptemp <- next.gen.p(sum(typeP)/sum(typeP+typep), wp)

		xh <- xhtemp
		xp <- xptemp

		xh <- mutation.h(mu.h, xh) # moved location of mutations
		xp <- mutation.p(mu.p, xp)
				
		xh <- recombination(c(xh), ra, rb, rc, R)
		
		nh <- matrix(rmultinom(1, size=num, prob=xh), ncol=8)
##		xp <- round(num * xp)
		
		output.h[i,] <- nh
		output.p[i] <- xp
		
		if(i==when.intro)
			break()
	}
	
	list(h=output.h[nrow(output.h),], p=output.p[nrow(output.p),], gen=i)
}


over.time <- function(nh.new, xp.new, mu.h, mu.p, alpha.h, alpha.p, beta, ra, rb, rc, R, num) {
#   frozen <- obtain.A(mu.h, mu.p, alpha.h, alpha.p, beta, ra, rb, rc, R, num)
#	nh <- frozen[[1]]
#	xp <- frozen[[2]]
#	nh.start <- frozen[[6]]
#	xp.start <- frozen[[5]]
#	nh.new <- nh
#	xp.new <- xp
	nh <- nh.new
	xp <- xp.new
	nh <- intro.b(nh)
	output.h <- matrix(nh, nrow=8)
	output.p <- matrix(xp)
	i <- 0
	
	repeat {
		i <- i + 1
		
		nh.old <- nh
		xp.old <- xp
		
		xh <- nh/num
##		xp <- xp/num
		
# Each host gets a single parasite, drawn from the effectively infinite pool of parasite propagules:
        	typeP <- rbinom(8,size=nh,prob=xp)
        	typep <- nh-typeP

#The frequency of P carried by each host genotype is
	    freqP <- typeP/(typeP+typep)

#Replacing NA with 0 for those host genotypes that don't exist
        freqP[is.na(freqP)] <- 0
 
#The average fitness of hosts of a particular genotype depends on the frequency of P actually within that genotype:
#Recall that matching reduces host fitness (alpha.h is negative).  With this haploid, single host-parasite locus model,
#MAM and IMAM can be obtained simply by switching the labels of H/h.

        	wh <- (1 + alpha.h * (gtypes$A * freqP + (1 - gtypes$A) * (1 -freqP))) * (1 + (gtypes$B) * beta)
		xhtemp <- next.gen.h(xh, wh)
		
#The fraction of parasites that match their host for both P and p parasites:

    	    matchP <- sum(gtypes$A*typeP)/sum(typeP)
        	matchp <- sum((1-gtypes$A)*typep)/sum(typep)

#Replacing NA with 0 for those parasite genotypes that don't exist
        matchP[is.na(matchP)] <- 0
        matchp[is.na(matchp)] <- 0
 
#Then the average fitness of each parasite genotype, accounting for the fitness advantage of infection via matching is:

        	wp <- 1 + alpha.p * c(matchP, matchp)

#These num parasites (one in each host) then produce an effectively infinite number of parasite propagules at frequency:

        xptemp <- next.gen.p(sum(typeP)/sum(typeP+typep), wp)
		
		xh <- xhtemp
		xp <- xptemp

		xh <- mutation.h(mu.h, xh) 
		xp <- mutation.p(mu.p, xp)
		
		xh <- recombination(c(xh), ra, rb, rc, R)
		
		nh <- matrix(rmultinom(1, size=num, prob=xh), nrow=8)
##		xp <- round(num * xp)

		output.h <- cbind(output.h, nh)
		output.p <- cbind(output.p, xp)
			
		total.b <- sum(c(nh[2], nh[4], nh[6], nh[8]))
		total.B <- sum(c(nh[1], nh[3], nh[5], nh[7]))
	
		if(total.b==0)
			break()
		if(total.B==0)
			break()		
	}	
	list(h=t(output.h), p=t(output.p), gen=i)	
}



multiple.runs <- function(nruns, nh.new, xp.new, f.A, f.B, f.M, num, mu.h, mu.p, alpha.p, alpha.h, beta, ra, rb, rc, R, gtypes, initialize){
	product.freq <- matrix(NA, nrow=nruns, ncol=8)
	product.M <- rep(NA, nruns)
	product.gen <- rep(NA, nruns)
	
	for(i in seq_len(nruns)){		
		poph.matrix <- matrix(over.time(nh.new,xp.new,mu.h,mu.p,alpha.h,alpha.p,beta,ra,rb,rc,R,num)$h, ncol=8)
		
		final.f <- poph.matrix[nrow(poph.matrix),]
		final.M <- sum(final.f[1:4])
		
		product.freq[i,] <- final.f
		product.M[i] <- final.M
		product.gen[i] <- nrow(poph.matrix)
	}
	matrix(c(product.freq, product.M, product.gen), ncol=10)
}


