# The following code is for creating replicates and multiple 
# runs of the simulation going through the initial burn-in, 
# and the repeat use of the burn-in frequencies of alleles
# with all of the tracking of different pieces of information
# that we want to retain from the runs 

do.one.rep <- function(nruns) {
  x <- runif(1)
  f.A <- c(x,1-x)
  nh <- c(initialize(f.A, f.B, f.M, num))
  xp <- runif(1)

# The burn-in population is created
  pop.A <- obtain.A(nh, xp, mu.h, mu.p, alpha.h, alpha.p, beta, ra,
                    rb, rc, R,  num)

  nh.new <- pop.A[[1]]
  xp.new <- pop.A[[2]]

# Multiple runs using the burn-in population:
  repeats <- multiple.runs(nruns, nh.new, xp.new, f.A, f.B, f.M, num,
                           mu.h,     mu.p, alpha.p, alpha.h, beta, ra,
                           rb, rc, R, gtypes, initialize)

# Formulas to track different pieces of information from each run
  total.B <- rowSums(repeats) - repeats[,10] - repeats[,9] -
    repeats[,2] - repeats[,4] - repeats[,6] - repeats[,8] 

  total.M <- rowSums(repeats) - repeats[,10] - repeats[,9] -
    repeats[,5] - repeats[,6] - repeats[,7] - repeats[,8]

  M.if.B <- ifelse(total.B==0, NA, total.M)
  M.if.b <- ifelse(total.B==0, total.M, NA)

  meanMB <- mean(M.if.B, na.rm = TRUE)
  meanMb <- mean(M.if.b, na.rm = TRUE)
  meanM <- mean(total.M, na.rm = TRUE)
  
  no.B.fix <- sum(total.B)
  prob.b.fix <- (no.B.fix/nruns)/num

  gen <- mean(repeats[,10])

  mean.matrix <- matrix(c(meanMB, meanMb, prob.b.fix, gen, meanM), ncol=5)
  mean.matrix
} 

# The code for running the above function
replicates <- function(nreps, nruns) {
  output <- matrix(NA, nrow=nreps, ncol=5)
  for ( i in seq_len(nreps) ) {
    mean.matrix <- try(do.one.rep(nruns))
   
    if( inherits(mean.matrix, "try-error") )
    output[i,] <- NA

    output[i,] <- mean.matrix
  }
  output
}
